#! /usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Cookies;
use Term::ReadKey;
use File::Basename qw(basename dirname);
use File::Copy;
use Getopt::Long;
use POSIX qw(strftime);
use Data::Dumper;

# Parse command line options
my $no_acls = 0;
my $update = 0;
Getopt::Long::Configure("bundling");
GetOptions(
    'fast|f'         => \$no_acls,
    'update|u'       => \$update,
    'help|h'         =>
        sub {
            print STDERR <<USAGE;
Usage: $0 [-hfu] <mediawiki_file>[, <mediawiki_file>]*
Options can be:
  --update, -u:                      update even if the page has been update by someone else
  --fast, -f:                        fast mode without changing ACLs
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);

# Get user authentification information
print "Enter login:\n";
my $user = <STDIN>;
chomp $user;
print "Enter password:\n";
ReadMode 'noecho';
my $passwd = ReadLine 0;
chomp $passwd;
ReadMode 'normal';

# Create a user agent object
my $ua = LWP::UserAgent->new(env_proxy => 1);
$ua->agent("$0 (gibus perl script to remotely insert a page into http://www.laquadrature.net/wiki/)");
$ua->cookie_jar({});
  
# Authentification stored in cookie
my $auth = HTTP::Request->new(POST => "http://www.laquadrature.net/wiki/index.php?title=Special:Userlogin&action=submitlogin&type=login&returnto=Accueil");
$auth->content_type('application/x-www-form-urlencoded');
$auth->content("wpName=$user&wpPassword=$passwd");
my $res_auth = $ua->request($auth);
unless (($res_auth->is_success or $res_auth->is_redirect) and $res_auth->content eq '') {
    # Hack Captcha
    if ($res_auth->content =~ /<label for="wpCaptchaWord">([^<]+)<\/label>/) {
        my $captcha = $1;
        my $res_captcha = eval $captcha;
        (my $captcha_id) = ($res_auth->content =~ /id="wpCaptchaId" value="([^"]+)"/);
        warn "Wrong user login/password retrying with captcha=!$captcha! !$res_captcha! captchaid=!$captcha_id!.\n";
        my $auth_captcha = HTTP::Request->new(POST => "http://www.laquadrature.net/wiki/index.php?title=Special:Userlogin&action=submitlogin&type=login&returnto=Accueil");
        $auth_captcha->content("wpName=$user&wpPassword=$passwd&wpCaptchaWord=$res_captcha&wpCaptchaId=$captcha_id");
        my $res_auth_captcha = $ua->request($auth_captcha);
        unless (($res_auth_captcha->is_success or $res_auth_captcha->is_redirect) and $res_auth_captcha->content eq '') {
            die "Wrong user login/password !".$res_auth_captcha->content."!.\n";
        }
    }
    die "Wrong user login/password !".$res_auth->content."!.\n";
}
print "OK identified\n";

foreach my $file (@ARGV) {
    # Define page to insert
    my $title = basename($file);
    $title =~ s/\.mediawiki$/\/fr/o;

    my $body;
    # Slurp file
    {
        local $/ = undef;
        open FILE, $file or die "Cannot open $file: $!\n";
        $body = <FILE>;
        close FILE;
    }

    # First request to display form in order to fetch hidden parameters.
    my $url = "http://www.laquadrature.net/wiki/index.php?title=$title&action=edit";
    my $req = HTTP::Request->new(POST => $url);
    $req->content_type('application/x-www-form-urlencoded');
    $req->content_length(0);
    my $res_first = $ua->request($req);
    unless ($res_first->is_success or $res_first->is_redirect) {
        die "Error in first request for !$url!: !".$res_first->message."! code=!".$res_first->content."!\n";
    }
    (my $wpEditToken) = $res_first->content =~ /type='hidden' value="([^"]*)" name="wpEditToken"/o;
    (my $wpEdittime) = $res_first->content =~ /type='hidden' value="([^"]*)" name="wpEdittime"/o;
    $wpEdittime = strftime("%Y%m%d%H%M%S",localtime(time-60)) unless $wpEdittime;
    #$wpEdittime = strftime("%Y%m%d%H%M%S",localtime) unless $wpEdittime;
    #$wpEdittime = '20080531162522' unless $wpEdittime;

    # Second request to insert the new page
    $url =~ s/action=edit/action=submit/;
    my $res = $ua->post($url, [
                                wpEdittime => $wpEdittime,
                                wpTextbox1 => $body,
                                wpEditToken => $wpEditToken,
                                wpWatchthis => 'true',
                                wpRecreate => 'true',
                              ]);
    unless (($res->is_success or $res->is_redirect) and $res->content eq '') {
        if (($res->is_success or $res->is_redirect) and $res->content =~ /<title>Conflit de modification/ and $update) {
            my $updater_wpStarttime = strftime("%Y%m%d%H%M%S",localtime(time-60));
            (my $updater_wpEdittime) = $res->content =~ /type='hidden' value="([^"]*)" name="wpEdittime"/o;
            (my $updater_wpEditToken) = $res->content =~ /type='hidden' value="([^"]*)" name="wpEditToken"/o;
            (my $updater_wpAutoSummary) = $res->content =~ /name="wpAutoSummary" type="hidden" value="([^"]*)"/o;
            (my $updater_wpTextbox2) = $res->content =~ /<textarea tabindex='6' id='wpTextbox2' name="wpTextbox2" rows='25' cols='80'>([^<]*)<\/textarea>/so;
            my $res_updater = $ua->post($url, [
                                        wpStarttime => $updater_wpStarttime,
                                        wpEdittime => $updater_wpEdittime,
                                        wpTextbox1 => $body,
                                        wpWatchthis => 'true',
                                        wpSave => 'Sauvegarder',
                                        wpEditToken => $updater_wpEditToken,
                                        wpAutoSummary => $updater_wpAutoSummary,
                                        wpTextbox2 => $updater_wpTextbox2,
                                      ]);
            unless (($res_updater->is_success or $res_updater->is_redirect) and $res_updater->content eq '') {
                die "Error in new updater request for !$url!: !".$res_updater->message."! code=!".$res_updater->code."! page=!".$res_updater->content."!\n";
            }
        }
        else {
            die "Error in second request for !$url!: !".$res->message."! code=!".$res->code."! page=!".$res->content."!\n";
        }
    }

    unless ($no_acls) {
        # Third request to protect page from editing by anonymous users
        $url =~ s/action=submit/action=protect/;
        my $res_protect = $ua->post($url, [
                                            "mwProtect-level-edit" => "autoconfirmed",
                                            "mwProtect-level-move" => "autoconfirmed",
                                            wpEditToken => $wpEditToken,
                                            mwProtectWatch => 'true',
                                          ]);
        unless (($res_protect->is_success or $res_protect->is_redirect) and $res_protect->content eq '') {
            die "Error in third request for !$url!: !".$res_protect->message."! code=!".$res_protect->code."! page=!".$res_protect->content."!\n";
        }
    }

    # Move page to uploaded directory
    (my $uploaded_dir = dirname($file)) =~ s/mediawiki/uploaded/;
    mkdir $uploaded_dir unless -d $uploaded_dir;
    move $file, $uploaded_dir;
    print "OK page $title inserted & moved to $uploaded_dir.\n";
}
